@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.96 2005/08/27 19:23:42 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.105 2006/01/16 21:29:40 crispin Exp $
  */
 
 #include "mozilla-config.h"
@@ -50,6 +50,8 @@
 #include "nsIDocShellTreeNode.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIWebPageDescriptor.h"
+#include "nsISHistory.h"
+#include "nsISHistoryInternal.h"
 #include "nsISHEntry.h"
 #include "nsIHistoryEntry.h"
 #include "nsIDOMHTMLDocument.h"
@@ -65,6 +67,7 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsIDOMNSEvent.h"
 #include "nsIDOMEventTarget.h"
+#include "nsIDOMNSEventTarget.h"
 #include "nsIDOMPopupBlockedEvent.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMElement.h"
@@ -81,6 +84,13 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMXMLDocument.h"
+#include <nsIDOMCSSStyleDeclaration.h>
+#include <nsIDOMCSSStyleDeclaration.h>
+#include <nsIDOMCSSPrimitiveValue.h>
+#include <nsIDOMCSSValue.h>
+#include <nsIDOMViewCSS.h>
+#include <nsIDOMDocumentView.h>
+#include <nsIDOMAbstractView.h>
 
 #ifdef ALLOW_PRIVATE_API
 #include "nsIImageDocument.h"
@@ -89,6 +99,7 @@
 /* will never be frozen */
 #include "nsIDocShell.h"
 #include "nsIMarkupDocumentViewer.h"
+#include <nsIDOMWindowInternal.h>
 #ifdef HAVE_MOZILLA_PSM
 /* not sure about this one: */
 #include <nsITransportSecurityInfo.h>
@@ -107,6 +118,7 @@ const static PRUnichar kDOMMouseScroll[]	 = { 'D', 'O', 'M', 'M', 'o', 'u', 's',
 const static PRUnichar kDOMPopupBlocked[]	 = { 'D', 'O', 'M', 'P', 'o', 'p', 'u', 'p', 'B', 'l', 'o', 'c', 'k', 'e', 'd', '\0' };
 const static PRUnichar kDOMWillOpenModalDialog[] = { 'D', 'O', 'M', 'W', 'i', 'l', 'l', 'O', 'p', 'e', 'n', 'M', 'o', 'd', 'a', 'l', 'D', 'i', 'a', 'l', 'o', 'g', '\0' };
 const static PRUnichar kDOMModalDialogClosed[]	 = { 'D', 'O', 'M', 'M', 'o', 'd', 'a', 'l', 'D', 'i', 'a', 'l', 'o', 'g', 'C', 'l', 'o', 's', 'e', 'd', '\0' };
+const static PRUnichar kDOMWindowClose[]	 = { 'D', 'O', 'M', 'W', 'i', 'n', 'd', 'o', 'w', 'C', 'l', 'o', 's', 'e', '\0' };
 const static PRUnichar kHrefAttr[]		 = { 'h', 'r', 'e', 'f', '\0' };
 const static PRUnichar kTypeAttr[]		 = { 't', 'y', 'p', 'e', '\0' };
 const static PRUnichar kTitleAttr[]		 = { 't', 'i', 't', 'l', 'e', '\0' };
@@ -117,13 +129,6 @@ NS_IMPL_ISUPPORTS1(EphyEventListener, nsIDOMEventListener)
 NS_IMETHODIMP
 EphyDOMLinkEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 {
-	/* make sure the event is trusted */
-	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
-	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
-	PRBool isTrusted = PR_FALSE;
-	nsEvent->GetIsTrusted (&isTrusted);
-	if (!isTrusted) return NS_OK;
-
 	nsCOMPtr<nsIDOMEventTarget> eventTarget;
 	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
 
@@ -183,7 +188,13 @@ EphyDOMLinkEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 		/* Only proceed for http favicons. Bug #312291 */
 		PRBool isHttp = PR_FALSE;
 		favUri->SchemeIs ("http", &isHttp);
+#ifdef HAVE_GECKO_1_8
+		PRBool isHttps = PR_FALSE;
+		favUri->SchemeIs ("https", &isHttps);
+		if (!isHttp && !isHttps) return NS_OK;
+#else
 		if (!isHttp) return NS_OK;
+#endif
 
 		/* check if load is allowed */
 		nsCOMPtr<nsIScriptSecurityManager> secMan
@@ -279,10 +290,8 @@ EphyDOMLinkEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 }
 
 NS_IMETHODIMP
-EphyDOMContentLoadedEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
+EphyMiscDOMEventsListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 {
-	LOG ("DOMContentLoaded event fired up");
-
 	/* make sure the event is trusted */
 	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
 	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
@@ -290,7 +299,30 @@ EphyDOMContentLoadedEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 	nsEvent->GetIsTrusted (&isTrusted);
 	if (!isTrusted) return NS_OK;
 
-	g_signal_emit_by_name (mOwner->mEmbed, "dom_content_loaded", (gpointer)aDOMEvent);
+	nsresult rv;
+	nsEmbedString type;
+	rv = aDOMEvent->GetType (type);
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	nsEmbedCString cType;
+	NS_UTF16ToCString (type, NS_CSTRING_ENCODING_UTF8, cType);
+
+	if (g_ascii_strcasecmp (cType.get(), "DOMContentLoaded") == 0)
+	{
+		g_signal_emit_by_name (mOwner->mEmbed, "dom_content_loaded",
+				       (gpointer)aDOMEvent);
+	}
+	else if (g_ascii_strcasecmp (cType.get(), "DOMWindowClose") == 0)
+	{
+		gboolean prevent = FALSE;
+
+		g_signal_emit_by_name (mOwner->mEmbed, "close-request", &prevent);
+
+		if (prevent)
+		{
+			aDOMEvent->PreventDefault ();
+		}
+	}
 
 	return NS_OK;
 }
@@ -320,13 +352,6 @@ EphyDOMLinkEventListener::GetDocURI (nsIDOMElement *aElement,
 NS_IMETHODIMP
 EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 {
-	/* make sure the event is trusted */
-	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
-	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
-	PRBool isTrusted = PR_FALSE;
-	nsEvent->GetIsTrusted (&isTrusted);
-	if (!isTrusted) return NS_OK;
-
 	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
 		do_QueryInterface (aDOMEvent);
 	NS_ENSURE_TRUE (popupEvent, NS_ERROR_FAILURE);
@@ -349,8 +374,20 @@ EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 			   NS_CSTRING_ENCODING_UTF8,
 			   popupWindowFeaturesString);
 
-	g_signal_emit_by_name(mOwner->mEmbed, "ge_popup_blocked",
+	nsEmbedCString popupWindowNameString;
+#ifdef HAVE_GECKO_1_9
+	nsEmbedString popupWindowName;
+	rv = popupEvent->GetPopupWindowName (popupWindowName);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	NS_UTF16ToCString (popupWindowName,
+			   NS_CSTRING_ENCODING_UTF8,
+			   popupWindowNameString);
+#endif
+
+	g_signal_emit_by_name(mOwner->mEmbed, "ge-popup-blocked",
 			      popupWindowURIString.get(),
+			      popupWindowNameString.get(),
 			      popupWindowFeaturesString.get());
 
 	return NS_OK;
@@ -361,13 +398,6 @@ EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 {
 	NS_ENSURE_TRUE (mOwner, NS_ERROR_FAILURE);
 
-	/* make sure the event is trusted */
-	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
-	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
-	PRBool isTrusted = PR_FALSE;
-	nsEvent->GetIsTrusted (&isTrusted);
-	if (!isTrusted) return NS_OK;
-
 	nsresult rv;
 	nsEmbedString type;
 	rv = aDOMEvent->GetType (type);
@@ -401,14 +431,7 @@ EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 NS_IMETHODIMP
 EphyDOMScrollEventListener::HandleEvent (nsIDOMEvent * aEvent)
 {
-	/* make sure the event is trusted */
 	nsresult rv;
-	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aEvent, &rv));
-	NS_ENSURE_SUCCESS (rv, rv);
-	PRBool isTrusted = PR_FALSE;
-	nsEvent->GetIsTrusted (&isTrusted);
-	if (!isTrusted) return NS_OK;
-
 	nsCOMPtr<nsIDOMMouseEvent> mouseEvent (do_QueryInterface (aEvent, &rv));
 	NS_ENSURE_SUCCESS (rv, rv);
 		
@@ -518,7 +541,7 @@ EphyContextMenuListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 
 EphyBrowser::EphyBrowser ()
 : mDOMLinkEventListener(nsnull)
-, mDOMContentLoadedEventListener(nsnull)
+, mMiscDOMEventsListener(nsnull)
 , mDOMScrollEventListener(nsnull)
 , mPopupBlockEventListener(nsnull)
 , mModalAlertListener(nsnull)
@@ -558,8 +581,8 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 	mDOMLinkEventListener = new EphyDOMLinkEventListener(this);
 	if (!mDOMLinkEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	mDOMContentLoadedEventListener = new EphyDOMContentLoadedEventListener(this);
-	if (!mDOMContentLoadedEventListener) return NS_ERROR_OUT_OF_MEMORY;
+	mMiscDOMEventsListener = new EphyMiscDOMEventsListener(this);
+	if (!mMiscDOMEventsListener) return NS_ERROR_OUT_OF_MEMORY;
 
 	mDOMScrollEventListener = new EphyDOMScrollEventListener(this);
 	if (!mDOMScrollEventListener) return NS_ERROR_OUT_OF_MEMORY;
@@ -658,21 +681,26 @@ EphyBrowser::AttachListeners(void)
 	NS_ENSURE_TRUE (mEventTarget, NS_ERROR_FAILURE);
 
 	nsresult rv;
-	rv = mEventTarget->AddEventListener(nsEmbedString(kDOMLinkAdded),
-					    mDOMLinkEventListener, PR_FALSE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMContentLoaded),
-					     mDOMContentLoadedEventListener, PR_FALSE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMMouseScroll),
-					     mDOMScrollEventListener, PR_TRUE); /* capture */
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMPopupBlocked),
-					     mPopupBlockEventListener, PR_FALSE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMWillOpenModalDialog),
-					     mModalAlertListener, PR_TRUE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMModalDialogClosed),
-					     mModalAlertListener, PR_TRUE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kContextMenu),
-					     mContextMenuListener, PR_TRUE /* capture */);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIDOMNSEventTarget> target (do_QueryInterface (mEventTarget, &rv));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	rv = target->AddEventListener(nsEmbedString(kDOMLinkAdded),
+				      mDOMLinkEventListener, PR_FALSE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMContentLoaded),
+				       mMiscDOMEventsListener, PR_FALSE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMWindowClose),
+				       mMiscDOMEventsListener, PR_FALSE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMMouseScroll),
+				       mDOMScrollEventListener, PR_TRUE /* capture */, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMPopupBlocked),
+				       mPopupBlockEventListener, PR_FALSE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMWillOpenModalDialog),
+				       mModalAlertListener, PR_TRUE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kDOMModalDialogClosed),
+				       mModalAlertListener, PR_TRUE, PR_FALSE);
+	rv |= target->AddEventListener(nsEmbedString(kContextMenu),
+				       mContextMenuListener, PR_TRUE /* capture */, PR_FALSE);
+	NS_ENSURE_SUCCESS (rv, rv);
 
 	return NS_OK;
 }
@@ -686,7 +714,9 @@ EphyBrowser::DetachListeners(void)
 	rv = mEventTarget->RemoveEventListener(nsEmbedString(kDOMLinkAdded),
 					       mDOMLinkEventListener, PR_FALSE);
 	rv |= mEventTarget->RemoveEventListener(nsEmbedString(kDOMContentLoaded),
-					        mDOMContentLoadedEventListener, PR_FALSE);
+					        mMiscDOMEventsListener, PR_FALSE);
+	rv |= mEventTarget->RemoveEventListener(nsEmbedString(kDOMWindowClose),
+						mMiscDOMEventsListener, PR_FALSE);
 	rv |= mEventTarget->RemoveEventListener(nsEmbedString(kDOMMouseScroll),
 						mDOMScrollEventListener, PR_TRUE); /* capture */
 	rv |= mEventTarget->RemoveEventListener(nsEmbedString(kDOMPopupBlocked),
@@ -787,6 +817,62 @@ nsresult EphyBrowser::GetSHistory (nsISHistory **aSHistory)
 	return NS_OK;
 }
 
+nsresult EphyBrowser::CopySHistory (EphyBrowser *dest, PRBool copy_back,
+	                            PRBool copy_forward, PRBool copy_current)
+{
+	nsresult rv;
+	
+	nsCOMPtr<nsISHistory> h_src;
+	GetSHistory (getter_AddRefs(h_src));
+	NS_ENSURE_TRUE (h_src, NS_ERROR_FAILURE);
+
+	PRInt32 count, index;
+	h_src->GetCount (&count);
+	h_src->GetIndex (&index);
+
+	nsCOMPtr<nsISHistory> h_dest;
+	dest->GetSHistory (getter_AddRefs (h_dest));
+	NS_ENSURE_TRUE (h_dest, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsISHistoryInternal> hi_dest = do_QueryInterface (h_dest);
+	NS_ENSURE_TRUE (hi_dest, NS_ERROR_FAILURE);
+
+	if (count)
+	{
+		nsCOMPtr<nsIHistoryEntry> he;
+		nsCOMPtr<nsISHEntry> she, dhe;
+
+		for (PRInt32 i = (copy_back ? 0 : index + 1); 
+		     i < (copy_forward ? count : index + 1);
+		     i++) 
+		{
+			rv = h_src->GetEntryAtIndex (i, PR_FALSE,
+						     getter_AddRefs (he));
+			NS_ENSURE_SUCCESS (rv, rv);
+
+			she = do_QueryInterface (he);
+			NS_ENSURE_TRUE (she, NS_ERROR_FAILURE);
+			
+			rv = she->Clone(getter_AddRefs (dhe));
+			NS_ENSURE_SUCCESS (rv, rv);
+
+			rv = hi_dest->AddEntry (dhe, PR_TRUE);
+			NS_ENSURE_SUCCESS (rv, rv);
+		}
+		
+		if (copy_current)
+		{
+			nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (dest->mWebBrowser);
+			NS_ENSURE_TRUE (wn_dest, NS_ERROR_FAILURE);
+			
+			rv = wn_dest->GotoIndex(index);
+			if (!NS_SUCCEEDED(rv)) return NS_ERROR_FAILURE;
+		}
+	}
+
+	return NS_OK;
+}
+
 nsresult EphyBrowser::Destroy ()
 {
 	DetachListeners ();
@@ -849,12 +935,60 @@ nsresult EphyBrowser::GetZoom (float *aZoom)
 	return mdv->GetTextZoom (aZoom);
 }
 
-nsresult EphyBrowser::GetDocument (nsIDOMDocument **aDOMDocument)
+nsresult
+EphyBrowser::ScrollLines (PRInt32 aNumLines)
+{
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+
+	mWebBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	if (!DOMWindow)
+	{
+		DOMWindow = mDOMWindow;
+	}
+	NS_ENSURE_TRUE (DOMWindow, NS_ERROR_FAILURE);
+
+	return DOMWindow->ScrollByLines (aNumLines);
+}
+
+nsresult
+EphyBrowser::ScrollPages (PRInt32 aNumPages)
+{
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+
+	mWebBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	if (!DOMWindow)
+	{
+		DOMWindow = mDOMWindow;
+	}
+	NS_ENSURE_TRUE (DOMWindow, NS_ERROR_FAILURE);
+
+	return DOMWindow->ScrollByPages (aNumPages);
+}
+
+nsresult
+EphyBrowser::ScrollPixels (PRInt32 aDeltaX,
+			   PRInt32 aDeltaY)
+{
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+
+	mWebBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	if (!DOMWindow)
+	{
+		DOMWindow = mDOMWindow;
+	}
+	NS_ENSURE_TRUE (DOMWindow, NS_ERROR_FAILURE);
+
+	return DOMWindow->ScrollBy (aDeltaX, aDeltaY);
+}
+
+nsresult
+EphyBrowser::GetDocument (nsIDOMDocument **aDOMDocument)
 {
 	return mDOMWindow->GetDocument (aDOMDocument);
 }
 
-nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
+nsresult
+EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 {
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
@@ -1114,6 +1248,24 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 	nsCOMPtr<nsIDOMHTMLCollection> forms;
 	htmlDoc->GetForms (getter_AddRefs (forms));
 	if (!forms) return NS_OK; /* it's ok not to have any forms */
+	
+	nsCOMPtr<nsIDOMDocumentView> docView (do_QueryInterface (aDomDoc));
+	nsCOMPtr<nsIDOMViewCSS> defaultCSSView;
+	if (docView)
+	{
+		nsCOMPtr<nsIDOMAbstractView> defaultView;
+		docView->GetDefaultView (getter_AddRefs (defaultView));
+		defaultCSSView = do_QueryInterface (defaultView);
+	}
+
+	const PRUnichar visibilityLiteral[] = { 'v', 'i', 's', 'i', 'b', 'i', 'l', 'i', 't', 'y', '\0' };
+	const PRUnichar visibleLiteral[] = { 'v', 'i', 's', 'i', 'b', 'l', 'e', '\0' };
+	nsEmbedString visibilityAttr(visibilityLiteral);
+	nsEmbedString visibleAttr(visibleLiteral);
+	nsEmbedString EmptyString;
+	nsCOMPtr<nsIDOMCSSStyleDeclaration> computedStyle;
+	nsCOMPtr<nsIDOMCSSValue> cssValue;
+	nsCOMPtr<nsIDOMCSSPrimitiveValue> primitiveValue;
 
 	PRUint32 formNum;
 	forms->GetLength (&formNum);
@@ -1128,6 +1280,32 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 		nsCOMPtr<nsIDOMHTMLFormElement> formElement = do_QueryInterface (formNode);
 		if (!formElement) continue;
 
+		PRBool isVisible = PR_FALSE;
+		nsresult rv;
+		computedStyle = nsnull;
+		rv = defaultCSSView->GetComputedStyle (formElement, EmptyString,
+						       getter_AddRefs (computedStyle));
+		if (NS_SUCCEEDED (rv) && computedStyle)
+		{
+			rv = computedStyle->GetPropertyCSSValue(visibilityAttr, getter_AddRefs (cssValue));
+			if (NS_SUCCEEDED (rv) && cssValue)
+			{
+				nsEmbedString value;
+				rv = cssValue->GetCssText (value);
+				if (NS_SUCCEEDED (rv) && value.Length ())
+				{
+					/* what to do for "collapse" and "inherit" values? */
+					isVisible = CompareFormsText (value, visibleAttr);
+				}
+			}
+		}
+
+		if (!isVisible)
+		{
+			LOG ("Form node %p is invisible\n", formNode.get());
+			continue;
+		}
+		
 		nsCOMPtr<nsIDOMHTMLCollection> formElements;
 		formElement->GetElements (getter_AddRefs (formElements));
 		if (!formElements) continue;
@@ -1142,9 +1320,45 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			formElements->Item (elementIndex, getter_AddRefs (domNode));
 			if (!domNode) continue;
 
+			nsCOMPtr<nsIDOMElement> domElement (do_QueryInterface (domNode));
+			if (!domElement) continue;
+
+			isVisible = PR_FALSE;
+			computedStyle = nsnull;
+			rv = defaultCSSView->GetComputedStyle (domElement, EmptyString,
+							       getter_AddRefs (computedStyle));
+			if (NS_SUCCEEDED (rv) && computedStyle)
+			{
+				rv = computedStyle->GetPropertyCSSValue(visibilityAttr, getter_AddRefs (cssValue));
+				if (NS_SUCCEEDED (rv) && cssValue)
+				{
+					nsEmbedString value;
+					rv = cssValue->GetCssText (value);
+					if (NS_SUCCEEDED (rv) && value.Length ())
+					{
+						/* what to do for "collapse" and "inherit" values? */
+						isVisible = CompareFormsText (value, visibleAttr);
+					}
+				}
+			}
+
+			if (!isVisible)
+			{
+				LOG("Form node %p element %p is invisible\n", formNode.get(), domNode.get());
+				continue;
+			}
+	
 			nsCOMPtr<nsIDOMHTMLTextAreaElement> areaElement = do_QueryInterface (domNode);
 			if (areaElement)
 			{
+				PRBool isDisabled = PR_TRUE;
+				areaElement->GetDisabled (&isDisabled);
+				if (isDisabled)
+				{
+					LOG ("Form node %p element %p [textarea] is disabled\n", formNode.get(), areaElement.get());
+					continue;
+				}
+
 				nsEmbedString defaultText, userText;
 				areaElement->GetDefaultValue (defaultText);
 				areaElement->GetValue (userText);
@@ -1161,6 +1375,14 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(domNode);
 			if (!inputElement) continue;
 	
+			PRBool isDisabled = PR_TRUE;
+			inputElement->GetDisabled (&isDisabled);
+			if (isDisabled)
+			{
+				LOG ("Form node %p element %p [input] is disabled\n", formNode.get(), inputElement.get());
+				continue;
+			}
+
 			nsEmbedString type;
 			inputElement->GetType(type);
 
@@ -1329,6 +1551,15 @@ EphyBrowser::GetDocumentType ()
   return type;
 }
 
+nsresult
+EphyBrowser::Close ()
+{
+	nsCOMPtr<nsIDOMWindowInternal> domWin (do_QueryInterface (mDOMWindow));
+	NS_ENSURE_TRUE (domWin, NS_ERROR_FAILURE);
+
+	return domWin->Close();
+}
+
 #ifndef HAVE_GECKO_1_8
 nsresult
 EphyBrowser::FocusActivate ()
