@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyPromptService.h,v 1.3.4.1 2006/02/10 14:34:14 chpe Exp $
+ *  $Id: EphyPromptService.h,v 1.1 2006/01/12 23:21:14 chpe Exp $
  */
 
 #ifndef EPHY_PROMPT_SERVICE_H
@@ -23,10 +23,6 @@
 
 #include <nsIPromptService.h>
 
-#if HAVE_NSINONBLOCKINGALERTSERVICE_H
-#include <nsINonBlockingAlertService.h>
-#endif
-
 #define EPHY_PROMPT_SERVICE_IID				\
 { /* 6e8b90d4-78a6-41c5-98da-b1559a40d30d */		\
   0x6e8b90d4, 0x78a6, 0x41c5,				\
@@ -35,17 +31,11 @@
 #define EPHY_PROMPT_SERVICE_CLASSNAME	"Epiphany Prompt Service"
 
 class EphyPromptService : public nsIPromptService
-#if HAVE_NSINONBLOCKINGALERTSERVICE_H
-			, public nsINonBlockingAlertService
-#endif
 {
 public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSIPROMPTSERVICE
-#if HAVE_NSINONBLOCKINGALERTSERVICE_H
-	NS_DECL_NSINONBLOCKINGALERTSERVICE
-#endif
-
+	
 	EphyPromptService();
 	virtual ~EphyPromptService();
 };
