@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-find.cpp,v 1.4 2005/08/15 22:22:31 chpe Exp $
+ *  $Id: mozilla-embed-find.cpp,v 1.6 2005/12/04 21:52:04 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -62,7 +62,7 @@ impl_set_properties (EphyEmbedFind *efind,
 	priv->find->SetFindProperties (find_string, case_sensitive);
 }
 
-static gboolean
+static EphyEmbedFindResult
 impl_find (EphyEmbedFind *efind,
 	     const char *find_string,
 	     gboolean links_only)
@@ -73,7 +73,7 @@ impl_find (EphyEmbedFind *efind,
 	return priv->find->Find (find_string, links_only);
 }
 
-static gboolean
+static EphyEmbedFindResult
 impl_find_again (EphyEmbedFind *efind,
 		   gboolean forward)
 {
@@ -83,6 +83,16 @@ impl_find_again (EphyEmbedFind *efind,
 	return priv->find->FindAgain (forward);
 }
 
+static void
+impl_set_selection (EphyEmbedFind *efind,
+		    gboolean attention)
+{
+	MozillaEmbedFind *find = MOZILLA_EMBED_FIND (efind);
+	MozillaEmbedFindPrivate *priv = find->priv;
+
+	priv->find->SetSelectionAttention (attention);
+}
+
 static gboolean
 impl_activate_link (EphyEmbedFind *efind,
 		    GdkModifierType mask)
@@ -100,6 +110,7 @@ ephy_find_iface_init (EphyEmbedFindIface *iface)
 	iface->set_properties = impl_set_properties;
 	iface->find = impl_find;
 	iface->find_again = impl_find_again;
+	iface->set_selection = impl_set_selection;
 	iface->activate_link = impl_activate_link;
 }
 
