@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: egg-editable-toolbar.c,v 1.78 2005/12/16 16:18:08 csaavedr Exp $
+ *  $Id: egg-editable-toolbar.c,v 1.83 2006/01/30 22:34:39 chpe Exp $
  */
 
 #include "config.h"
@@ -32,6 +32,7 @@
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtktoggleaction.h>
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
@@ -48,11 +49,9 @@
 
 static void egg_editable_toolbar_class_init	(EggEditableToolbarClass *klass);
 static void egg_editable_toolbar_init		(EggEditableToolbar *etoolbar);
-static void egg_editable_toolbar_finalize	(GObject *object);
 
 #define MIN_TOOLBAR_HEIGHT 20
 #define EGG_ITEM_NAME      "egg-item-name"
-#define EGG_TOOLITEM       "egg-toolitem"
 
 static const GtkTargetEntry dest_drag_types[] = {
   {EGG_TOOLBAR_ITEM_TYPE, GTK_TARGET_SAME_APP, 0},
@@ -62,7 +61,9 @@ enum
 {
   PROP_0,
   PROP_TOOLBARS_MODEL,
-  PROP_UI_MANAGER
+  PROP_UI_MANAGER,
+  PROP_VISIBILITY_PATH,
+  PROP_SELECTED
 };
 
 enum
@@ -84,6 +85,14 @@ struct _EggEditableToolbarPrivate
   guint edit_mode;
   gboolean save_hidden;
   GtkWidget *fixed_toolbar;
+  
+  GtkWidget *selected;
+  GtkActionGroup *actions;
+  guint popup_id;
+  
+  guint visibility_id;
+  char *visibility_path;
+  GPtrArray *visibility_actions;
 
   guint        dnd_pending;
   GtkToolbar  *dnd_toolbar;
@@ -118,7 +127,8 @@ egg_editable_toolbar_get_type (void)
 }
 
 static int
-get_dock_position (EggEditableToolbar *etoolbar, GtkWidget *dock)
+get_dock_position (EggEditableToolbar *etoolbar,
+                   GtkWidget *dock)
 {
   GList *l;
   int result;
@@ -228,8 +238,6 @@ drag_begin_cb (GtkWidget          *widget,
 	       GdkDragContext     *context,
 	       EggEditableToolbar *etoolbar)
 {
-  widget = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
-  g_return_if_fail (widget != NULL);
   gtk_widget_hide (widget);
 }
 
@@ -238,8 +246,6 @@ drag_end_cb (GtkWidget          *widget,
 	     GdkDragContext     *context,
 	     EggEditableToolbar *etoolbar)
 {
-  widget = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
-  g_return_if_fail (widget != NULL);
   gtk_widget_show (widget);
 }
 
@@ -255,9 +261,6 @@ drag_data_get_cb (GtkWidget          *widget,
   const char *name;
   char *data;
 
-  widget = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
-  g_return_if_fail (widget != NULL);
-  
   g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (etoolbar));
   model = egg_editable_toolbar_get_model (etoolbar);
   
@@ -277,58 +280,37 @@ drag_data_get_cb (GtkWidget          *widget,
 }
 
 static void
-move_item_cb (GtkWidget          *menuitem,
+move_item_cb (GtkAction          *action,
               EggEditableToolbar *etoolbar)
 {
-  GtkWidget *toolitem = g_object_get_data (G_OBJECT (menuitem), EGG_TOOLITEM);
+  GtkWidget *toolitem = gtk_widget_get_ancestor (egg_editable_toolbar_get_selected (etoolbar), GTK_TYPE_TOOL_ITEM);
   GtkTargetList *list = gtk_target_list_new (dest_drag_types, G_N_ELEMENTS (dest_drag_types));
-  gtk_drag_begin (toolitem, list, GDK_ACTION_MOVE, 1, NULL);
+
+  GdkEvent *realevent = gtk_get_current_event();
+  GdkEventMotion event;
+  event.type = GDK_MOTION_NOTIFY;
+  event.window = realevent->any.window;
+  event.send_event = FALSE;
+  event.axes = NULL;
+  event.time = gdk_event_get_time (realevent);
+  gdk_event_get_state (realevent, &event.state);
+  gdk_event_get_coords (realevent, &event.x, &event.y);
+  gdk_event_get_root_coords (realevent, &event.x_root, &event.y_root);
+    
+  gtk_drag_begin (toolitem, list, GDK_ACTION_MOVE, 1, (GdkEvent *)&event);
   gtk_target_list_unref (list);
 }
 
 static void
-set_dock_visible (EggEditableToolbar *etoolbar,
-                  GtkWidget          *dock,
-                  gboolean            visible)
-{
-  if (visible)
-    {
-      gtk_widget_show (dock);
-    }
-  else
-    {
-      gtk_widget_hide (dock);
-    }
-  
-  if (etoolbar->priv->save_hidden)
-    {
-      int position = get_dock_position (etoolbar, dock);
-      EggTbModelFlags flags = egg_toolbars_model_get_flags
-        (etoolbar->priv->model, position);
-      
-      if (visible)
-        {
-	  flags &= ~(EGG_TB_MODEL_HIDDEN);
-	}
-      else
-	{
-	  flags |=  (EGG_TB_MODEL_HIDDEN);
-	}
-      
-      egg_toolbars_model_set_flags (etoolbar->priv->model, position, flags);
-    }
-}
-
-static void
-remove_item_cb (GtkWidget          *menuitem,
+remove_item_cb (GtkAction          *action,
                 EggEditableToolbar *etoolbar)
 {
-  GtkWidget *toolitem = g_object_get_data (G_OBJECT (menuitem), EGG_TOOLITEM);
+  GtkWidget *toolitem = gtk_widget_get_ancestor (egg_editable_toolbar_get_selected (etoolbar), GTK_TYPE_TOOL_ITEM);
   int pos, toolbar_pos;
       
   toolbar_pos = get_toolbar_position (etoolbar, toolitem->parent);
-  pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolitem->parent),
-                                    GTK_TOOL_ITEM (toolitem));
+  pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolitem->parent), 
+				    GTK_TOOL_ITEM (toolitem));
 
   egg_toolbars_model_remove_item (etoolbar->priv->model,
 			          toolbar_pos, pos);
@@ -340,205 +322,17 @@ remove_item_cb (GtkWidget          *menuitem,
 }
 
 static void
-remove_toolbar_cb (GtkWidget          *menuitem,
+remove_toolbar_cb (GtkAction          *action,
 		   EggEditableToolbar *etoolbar)
 {
-  GtkWidget *toolbar = g_object_get_data (G_OBJECT (menuitem), "egg-toolbar");
+  GtkWidget *selected = egg_editable_toolbar_get_selected (etoolbar);
+  GtkWidget *toolbar = gtk_widget_get_ancestor (selected, GTK_TYPE_TOOLBAR);
   int toolbar_pos;
 
   toolbar_pos = get_toolbar_position (etoolbar, toolbar);
   egg_toolbars_model_remove_toolbar (etoolbar->priv->model, toolbar_pos);
 }
 
-static void
-toggle_visibility_cb (GtkWidget          *menuitem,
-                      EggEditableToolbar *etoolbar)
-{
-  GtkWidget *dock = g_object_get_data (G_OBJECT (menuitem), "egg-dock");
-  set_dock_visible (etoolbar, dock, !GTK_WIDGET_VISIBLE (dock));
-}
-
-static void
-egg_editable_toolbar_add_visibility_items (EggEditableToolbar *etoolbar,
-                                           GtkMenu            *popup)
-{
-  EggToolbarsModel *model = etoolbar->priv->model;    
-  GtkCheckMenuItem *item;
-  GtkWidget *dock;
-  int n_toolbars, n_items, n_visible = 0;
-  int i, j, k, l;
-
-  g_return_if_fail (model != NULL);
-  g_return_if_fail (etoolbar->priv->manager != NULL);
-
-  n_toolbars = egg_toolbars_model_n_toolbars (model);
-
-  for (i = 0; i < n_toolbars; i++)
-    {
-      dock = get_dock_nth (etoolbar, i);
-      if (GTK_WIDGET_VISIBLE (dock))
-        n_visible++;
-    }
-  
-  if (GTK_MENU_SHELL(popup)->children != NULL)
-    {
-      GtkWidget *separator = gtk_separator_menu_item_new ();
-      gtk_widget_show (separator);
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator);
-    }
-        
-  for (i = 0; i < n_toolbars; i++)
-    {
-      char buffer[40] = "Empty";
-
-      n_items = egg_toolbars_model_n_items (model, i);
-      for (k = 0, j = 0; j < n_items && k < sizeof(buffer)-1; j++)
-        {
-          GValue value = { 0, };
-          GtkAction *action;
-          const char *name;
-
-          name = egg_toolbars_model_item_nth (model, i, j);
-          if (name == NULL) continue;
-          action = find_action (etoolbar, name);
-          if (action == NULL) continue;
-
-          g_value_init (&value, G_TYPE_STRING);
-          g_object_get_property (G_OBJECT (action), "label", &value);
-          name = g_value_get_string (&value);
-          if (name == NULL) continue;
-            
-          if (j > 0)
-            {
-                if(k<sizeof(buffer)-1) buffer[k++] = ',';
-                if(k<sizeof(buffer)-1) buffer[k++] = ' ';
-            }
-          
-          for (l = 0; name[l] && k<sizeof(buffer)-1; l++)
-            {
-                switch(name[l])
-                {
-                 case '_':
-                 case '.':
-                 case ',':
-                    break;
-                 default:
-                    buffer[k++] = name[l];
-                }
-            }
-            
-          if (name[l])
-            {
-                l = k-5;
-                while(l>0 && buffer[l] != ',') l--;
-                if(buffer[l] == ',') k = l + 2;
-                else k = k-3;
-                
-                buffer[k++] = '.';
-                buffer[k++] = '.';
-                buffer[k++] = '.';
-                buffer[k] = 0;
-                break;
-            }
-            
-          buffer[k] = 0;
-
-          g_value_unset (&value);
-        }
-      
-                
-      dock = get_dock_nth (etoolbar, i);
-      item = GTK_CHECK_MENU_ITEM (gtk_check_menu_item_new_with_label (buffer));
-      gtk_check_menu_item_set_active (item, GTK_WIDGET_VISIBLE (dock));
-      gtk_widget_set_sensitive (GTK_WIDGET (item), (n_visible > 1 || !GTK_WIDGET_VISIBLE (dock)));
-      gtk_widget_show (GTK_WIDGET (item));
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), GTK_WIDGET (item));       
-
-      g_object_set_data (G_OBJECT (item), "egg-dock", dock);
-      g_signal_connect (item, "toggled",
-                        G_CALLBACK (toggle_visibility_cb),
-                        etoolbar);
-    }
-}
-
-void
-egg_editable_toolbar_add_popup_items (GtkWidget *widget,
-                                      GtkMenu   *popup)
-{
-  EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR
-      (gtk_widget_get_ancestor (widget, EGG_TYPE_EDITABLE_TOOLBAR));
-  GtkWidget *toolbar  = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOLBAR);
-  GtkWidget *toolitem = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
-  GtkWidget *item, *image;
-  int separated;
-    
-  separated = (GTK_MENU_SHELL(popup)->children == NULL);
-    
-  if (etoolbar != NULL && toolitem != NULL)
-    {
-      if (!separated)
-        {
-          item = gtk_separator_menu_item_new ();
-          gtk_widget_show (item);
-          gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-          separated = 1;
-        }
-        
-      item = gtk_menu_item_new_with_mnemonic (_("_Move on Toolbar"));
-      g_object_set_data (G_OBJECT (item), EGG_TOOLITEM, toolitem);
-      gtk_widget_show (item);
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-      g_signal_connect (item, "activate",
-                        G_CALLBACK (move_item_cb),
-                        etoolbar);
-        
-      item = gtk_image_menu_item_new_with_mnemonic (_("_Remove from Toolbar"));
-      g_object_set_data (G_OBJECT (item), EGG_TOOLITEM, toolitem);
-      gtk_widget_show (item);
-      image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-      g_signal_connect (item, "activate",
-                        G_CALLBACK (remove_item_cb),
-                        etoolbar);
-    }
-
-  if (etoolbar != NULL && toolbar != NULL)
-    {
-      int position;
-      EggTbModelFlags flags;
-        
-      position = get_toolbar_position (etoolbar, toolbar);
-      flags = egg_toolbars_model_get_flags (etoolbar->priv->model, position);
-        
-      if (etoolbar->priv->edit_mode > 0 && (flags & EGG_TB_MODEL_NOT_REMOVABLE)==0)
-        {
-            if (!separated)
-            {
-                item = gtk_separator_menu_item_new ();
-                gtk_widget_show (item);
-                gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-                separated = 1;
-            }
-        
-            item = gtk_image_menu_item_new_with_mnemonic (_("_Remove Toolbar"));
-            g_object_set_data (G_OBJECT (item), "egg-toolbar", toolbar);
-            gtk_widget_show (item);
-            image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
-            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-            gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-            g_signal_connect (item, "activate",
-                              G_CALLBACK (remove_toolbar_cb),
-                              etoolbar);
-        }
-
-      if (egg_toolbars_model_n_toolbars (etoolbar->priv->model) > 1)
-        {
-            egg_editable_toolbar_add_visibility_items (etoolbar, popup);
-        }
-    }
-}
-
 static void
 popup_context_menu_cb (GtkWidget          *toolbar,
                        gint		   x,
@@ -546,10 +340,15 @@ popup_context_menu_cb (GtkWidget          *toolbar,
                        gint                button_number,
                        EggEditableToolbar *etoolbar)
 {
-  GtkMenu *menu = GTK_MENU (gtk_menu_new ());
-  egg_editable_toolbar_add_popup_items (toolbar, menu);
-  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, button_number,
-                  gtk_get_current_event_time ());
+  if (etoolbar->priv->popup_id != 0)
+    {
+      GtkMenu *menu;
+      egg_editable_toolbar_set_selected (etoolbar, toolbar);
+      g_object_notify (G_OBJECT (etoolbar), "selected");
+	
+      menu = GTK_MENU (gtk_ui_manager_get_widget (etoolbar->priv->manager, "/ToolbarPopup"));
+      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, button_number, gtk_get_current_event_time ());
+    }
 }
 
 static gboolean
@@ -557,12 +356,15 @@ button_press_event_cb (GtkWidget *widget,
                        GdkEventButton *event,
                        EggEditableToolbar *etoolbar)
 {
-  if (event->button == 3)
+  if (event->button == 3 && etoolbar->priv->popup_id != 0)
     {
-      GtkMenu *menu = GTK_MENU (gtk_menu_new ());
-      egg_editable_toolbar_add_popup_items (widget, menu);
-      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event->button,
-                      event->time);
+      GtkMenu *menu;
+      egg_editable_toolbar_set_selected (etoolbar, widget);
+      g_object_notify (G_OBJECT (etoolbar), "selected");
+	
+      menu = GTK_MENU (gtk_ui_manager_get_widget (etoolbar->priv->manager, "/ToolbarPopup"));
+      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event->button, event->time);
+      
       return TRUE;
     }
     
@@ -575,56 +377,44 @@ configure_item_sensitivity (GtkToolItem *item, EggEditableToolbar *etoolbar)
   GtkAction *action;
   char *name;
   
-  g_return_if_fail (etoolbar != NULL);
-  
-  if (etoolbar->priv->edit_mode > 0)
-    {
-      GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (item), GTK_SENSITIVE);
-      gtk_tool_item_set_use_drag_window (item, TRUE);
-      gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
-      return;
-    }
-  
   name = g_object_get_data (G_OBJECT (item), EGG_ITEM_NAME);
   action = name ? find_action (etoolbar, name) : NULL;
+  
+  if (action)
+    {
+      g_object_notify (G_OBJECT (action), "sensitive");
+    }
 
-  if (action != NULL && gtk_action_is_sensitive (action))
-    {
-      GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (item), GTK_SENSITIVE);
-      gtk_tool_item_set_use_drag_window (item, FALSE);
-      gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
-    }
-  else
-    {
-       gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
-       gtk_tool_item_set_use_drag_window (item, TRUE);
-       GTK_WIDGET_SET_FLAGS (GTK_WIDGET (item), GTK_SENSITIVE);
-    }
+  gtk_tool_item_set_use_drag_window (item,
+				     (etoolbar->priv->edit_mode > 0) || 
+				     GTK_IS_SEPARATOR_TOOL_ITEM (item));
+  
 }
 
 static void
-configure_item_cursor (GtkToolItem *item, EggEditableToolbar *etoolbar)
+configure_item_cursor (GtkToolItem *item,
+		       EggEditableToolbar *etoolbar)
 {
-  g_return_if_fail (etoolbar != NULL);
-  g_return_if_fail (GTK_WIDGET(item)->window != NULL);
-  
-  if (etoolbar->priv->edit_mode > 0)
-    {
-      GdkCursor *cursor;
-          
-      cursor = gdk_cursor_new (GDK_HAND2);
-      gdk_window_set_cursor (GTK_WIDGET(item)->window, cursor);
-      gdk_cursor_unref (cursor);
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  GtkWidget *widget = GTK_WIDGET (item);
 
-      gtk_drag_source_set (GTK_WIDGET (item), GDK_BUTTON1_MASK, dest_drag_types,
-                           G_N_ELEMENTS (dest_drag_types), GDK_ACTION_MOVE);
-    }
-  else
+  if (widget->window != NULL)
     {
-      gdk_window_set_cursor (GTK_WIDGET(item)->window, NULL);
-      
-      gtk_drag_source_set (GTK_WIDGET (item), GDK_BUTTON2_MASK, dest_drag_types,
-                           G_N_ELEMENTS (dest_drag_types), GDK_ACTION_MOVE);      
+      if (priv->edit_mode > 0)
+        {
+          GdkCursor *cursor;
+          
+          cursor = gdk_cursor_new (GDK_HAND2);
+          gdk_window_set_cursor (widget->window, cursor);
+          gdk_cursor_unref (cursor);
+
+          gtk_drag_source_set (widget, GDK_BUTTON1_MASK, dest_drag_types,
+                               G_N_ELEMENTS (dest_drag_types), GDK_ACTION_MOVE);
+        }
+      else
+        {
+          gdk_window_set_cursor (GTK_WIDGET(item)->window, NULL);
+        }
     }
 }
 
@@ -638,22 +428,27 @@ connect_widget_signals (GtkWidget *proxy, EggEditableToolbar *etoolbar)
                               (gpointer) etoolbar);
     }
 
+  if (GTK_IS_TOOL_ITEM (proxy))
+    {
+      g_signal_connect_object (proxy, "drag_begin",
+			       G_CALLBACK (drag_begin_cb), 
+			       etoolbar, 0);
+      g_signal_connect_object (proxy, "drag_end",
+			       G_CALLBACK (drag_end_cb),
+			       etoolbar, 0);
+      g_signal_connect_object (proxy, "drag_data_get",
+			       G_CALLBACK (drag_data_get_cb), 
+			       etoolbar, 0);
+      g_signal_connect_object (proxy, "drag_data_delete",
+			       G_CALLBACK (drag_data_delete_cb),
+			       etoolbar, 0);
+    }
+    
   if (GTK_IS_BUTTON (proxy) || GTK_IS_TOOL_ITEM (proxy))
     {
-      g_signal_connect (proxy, "drag_begin",
-                        G_CALLBACK (drag_begin_cb), etoolbar);
-      g_signal_connect (proxy, "drag_end",
-                        G_CALLBACK (drag_end_cb), etoolbar);
-      g_signal_connect (proxy, "drag_data_get",
-                        G_CALLBACK (drag_data_get_cb), etoolbar);
-      g_signal_connect (proxy, "drag_data_delete",
-                        G_CALLBACK (drag_data_delete_cb), etoolbar);
-      g_signal_connect (proxy, "drag_data_get",
-                        G_CALLBACK (drag_data_get_cb), etoolbar);
-      g_signal_connect (proxy, "button-press-event",
-                        G_CALLBACK (button_press_event_cb), etoolbar);
-      gtk_drag_source_set (proxy, GDK_BUTTON2_MASK, dest_drag_types,
-                           G_N_ELEMENTS (dest_drag_types), GDK_ACTION_MOVE);      
+      g_signal_connect_object (proxy, "button-press-event",
+			       G_CALLBACK (button_press_event_cb),
+			       etoolbar, 0);
     }
 }
 
@@ -664,7 +459,11 @@ action_sensitive_cb (GtkAction   *action,
 {
   EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR
     (gtk_widget_get_ancestor (GTK_WIDGET (item), EGG_TYPE_EDITABLE_TOOLBAR));
-  configure_item_sensitivity (item, etoolbar);
+
+  if (etoolbar->priv->edit_mode > 0)
+    {
+      gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
+    }
 }
 
 static GtkToolItem *
@@ -678,7 +477,6 @@ create_item_from_action (EggEditableToolbar *etoolbar,
   if (strcmp (name, "_separator") == 0)
     {
       item = gtk_separator_tool_item_new ();
-      gtk_tool_item_set_use_drag_window (item, TRUE);
     }
   else
     {
@@ -890,6 +688,161 @@ configure_drag_dest (EggEditableToolbar *etoolbar,
   }
 }
 
+static void
+toggled_visibility_cb (GtkToggleAction *action,
+		       EggEditableToolbar *etoolbar)
+{
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  GtkWidget *dock;
+  EggTbModelFlags flags;
+  gboolean visible;
+  gint i;
+  
+  visible = !gtk_toggle_action_get_active (action);
+  for (i = 0; i < priv->visibility_actions->len; i++)
+    if (g_ptr_array_index (priv->visibility_actions, i) == action)
+      break;
+  
+  g_return_if_fail (i < priv->visibility_actions->len);
+  
+  dock = get_dock_nth (etoolbar, i);  
+  if (visible)
+    {
+      gtk_widget_show (dock);
+    }
+  else
+    {
+      gtk_widget_hide (dock);
+    }
+  
+  if (priv->save_hidden)
+    {      
+      flags = egg_toolbars_model_get_flags (priv->model, i);
+      
+      if (visible)
+        {
+	  flags &= ~(EGG_TB_MODEL_HIDDEN);
+	}
+      else
+	{
+	  flags |=  (EGG_TB_MODEL_HIDDEN);
+	}
+      
+      egg_toolbars_model_set_flags (priv->model, i, flags);
+    }
+}
+
+static void
+toolbar_visibility_refresh (EggEditableToolbar *etoolbar)
+{
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  gint n_toolbars, n_items, i, j, k;
+  GtkToggleAction *action;
+  GString *string;
+  char action_name[40];
+  char *action_label;
+  char *tmp;    	
+  
+  if (priv == NULL || priv->model == NULL || priv->manager == NULL ||
+      priv->visibility_path == NULL || priv->actions == NULL)
+    {
+      return;
+    }
+
+  if (priv->visibility_actions == NULL)
+    {
+      priv->visibility_actions = g_ptr_array_new ();
+    }
+  
+  if (priv->visibility_id != 0)
+    {
+      gtk_ui_manager_remove_ui (priv->manager, priv->visibility_id);
+    }  
+  
+  priv->visibility_id = gtk_ui_manager_new_merge_id (priv->manager);
+  
+  n_toolbars = egg_toolbars_model_n_toolbars (priv->model);
+  for (i = 0; i < n_toolbars; i++)
+    {
+      string = g_string_sized_new (0);
+      n_items = egg_toolbars_model_n_items (priv->model, i);
+      for (k = 0, j = 0; j < n_items; j++)
+        {
+          GValue value = { 0, };
+          GtkAction *action;
+          const char *name;
+
+          name = egg_toolbars_model_item_nth (priv->model, i, j);
+          if (name == NULL) continue;
+          action = find_action (etoolbar, name);
+          if (action == NULL) continue;
+
+          g_value_init (&value, G_TYPE_STRING);
+          g_object_get_property (G_OBJECT (action), "label", &value);
+          name = g_value_get_string (&value);
+          if (name == NULL) continue;
+	  
+	  k += g_utf8_strlen (name, -1) + 2;
+	  if (j > 0)
+	    {
+	      g_string_append (string, ", ");
+	      if (k > 25) break;
+	    }
+	  g_string_append (string, name);
+	}
+      if (j < n_items)
+        {
+	  g_string_append (string, " ...");
+        }
+      
+      tmp = g_string_free (string, FALSE);
+      for (j = 0, k = 0; tmp[j]; j++)
+      {
+	if (tmp[j] == '_') continue;
+	tmp[k] = tmp[j];
+	k++;
+      }
+      tmp[k] = 0;
+      action_label = g_strdup_printf (_("Hide “%s”"), tmp);
+      g_free (tmp);
+      
+      sprintf(action_name, "ToolbarToggle%d", i);
+      
+      if (i >= priv->visibility_actions->len)
+        {
+	  action = gtk_toggle_action_new (action_name, action_label, NULL, NULL);
+	  g_ptr_array_add (priv->visibility_actions, action);
+	  g_signal_connect_object (action, "toggled",
+				   G_CALLBACK (toggled_visibility_cb),
+				   etoolbar, 0);
+	  gtk_action_group_add_action (priv->actions, GTK_ACTION (action));
+	}
+      else
+        {
+	  action = g_ptr_array_index (priv->visibility_actions, i);
+	  g_object_set (action, "label", action_label, NULL);
+        }
+      
+      gtk_toggle_action_set_active (action, !GTK_WIDGET_VISIBLE
+				    (get_dock_nth (etoolbar, i)));
+      
+      gtk_ui_manager_add_ui (priv->manager, priv->visibility_id,
+			     priv->visibility_path, action_name, action_name,
+			     GTK_UI_MANAGER_MENUITEM, FALSE);
+	    
+      g_free (action_label);
+    }
+  
+  gtk_ui_manager_ensure_update (priv->manager);
+  
+  while (i < priv->visibility_actions->len)
+    {
+      action = g_ptr_array_index (priv->visibility_actions, i);
+      g_ptr_array_remove_index_fast (priv->visibility_actions, i);
+      gtk_action_group_remove_action (priv->actions, GTK_ACTION (action));
+      i++;
+    }
+}
 
 static GtkWidget *
 create_dock (EggEditableToolbar *etoolbar)
@@ -978,6 +931,8 @@ toolbar_changed_cb (EggToolbarsModel   *model,
     {
       set_fixed_style (etoolbar, style);
     }
+
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
@@ -1035,6 +990,8 @@ toolbar_added_cb (EggToolbarsModel   *model,
   gtk_widget_show_all (dock);
   
   update_fixed (etoolbar);
+
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
@@ -1043,7 +1000,6 @@ toolbar_removed_cb (EggToolbarsModel   *model,
 	            EggEditableToolbar *etoolbar)
 {
   GtkWidget *dock;
-  int i;
 
   if (position == 0 && etoolbar->priv->fixed_toolbar != NULL)
     {
@@ -1053,19 +1009,9 @@ toolbar_removed_cb (EggToolbarsModel   *model,
   dock = get_dock_nth (etoolbar, position);
   gtk_widget_destroy (dock);
 
-  dock = NULL;
-  for (i = egg_toolbars_model_n_toolbars (model)-1; i >= 0; i--)
-    {
-      dock = get_dock_nth (etoolbar, i);
-      if (GTK_WIDGET_VISIBLE (dock)) break;
-    }
-      
-  if (i < 0 && dock != NULL)
-    {
-      set_dock_visible (etoolbar, dock, TRUE);
-    }
-    
   update_fixed (etoolbar);
+  
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
@@ -1091,6 +1037,8 @@ item_added_cb (EggToolbarsModel   *model,
   dock = get_dock_nth (etoolbar, tpos);
   gtk_widget_set_size_request (dock, -1, -1);
   gtk_widget_queue_resize_no_redraw (dock);
+
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
@@ -1099,6 +1047,8 @@ item_removed_cb (EggToolbarsModel   *model,
 	         int                 position,
 	         EggEditableToolbar *etoolbar)
 {
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  
   GtkWidget *toolbar;
   GtkWidget *item;
 
@@ -1106,11 +1056,19 @@ item_removed_cb (EggToolbarsModel   *model,
   item = GTK_WIDGET (gtk_toolbar_get_nth_item
 	(GTK_TOOLBAR (toolbar), position));
   g_return_if_fail (item != NULL);
+
+  if (item == priv->selected)
+    {
+      /* FIXME */
+    }
+
   gtk_container_remove (GTK_CONTAINER (toolbar), item);
+
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
-egg_editable_toolbar_construct (EggEditableToolbar *etoolbar)
+egg_editable_toolbar_build (EggEditableToolbar *etoolbar)
 {
   int i, l, n_items, n_toolbars;
   EggToolbarsModel *model = etoolbar->priv->model;
@@ -1203,46 +1161,130 @@ egg_editable_toolbar_deconstruct (EggEditableToolbar *toolbar)
 }
 
 void
-egg_editable_toolbar_set_model (EggEditableToolbar *toolbar,
+egg_editable_toolbar_set_model (EggEditableToolbar *etoolbar,
 				EggToolbarsModel   *model)
 {
-  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (model));
-  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (toolbar));
-  g_return_if_fail (toolbar->priv->manager);
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
 
-  if (toolbar->priv->model == model) return;
+  if (priv->model == model) return;
 
-  if (toolbar->priv->model)
+  if (priv->model)
     {
-      egg_editable_toolbar_disconnect_model (toolbar);
-      egg_editable_toolbar_deconstruct (toolbar);
+      egg_editable_toolbar_disconnect_model (etoolbar);
+      egg_editable_toolbar_deconstruct (etoolbar);
 
-      g_object_unref (toolbar->priv->model);
+      g_object_unref (priv->model);
     }
 
-  toolbar->priv->model = g_object_ref (model);
+  priv->model = g_object_ref (model);
 
-  egg_editable_toolbar_construct (toolbar);
+  egg_editable_toolbar_build (etoolbar);
+
+  toolbar_visibility_refresh (etoolbar);
 
   g_signal_connect (model, "item_added",
-		    G_CALLBACK (item_added_cb), toolbar);
+		    G_CALLBACK (item_added_cb), etoolbar);
   g_signal_connect (model, "item_removed",
-		    G_CALLBACK (item_removed_cb), toolbar);
+		    G_CALLBACK (item_removed_cb), etoolbar);
   g_signal_connect (model, "toolbar_added",
-		    G_CALLBACK (toolbar_added_cb), toolbar);
+		    G_CALLBACK (toolbar_added_cb), etoolbar);
   g_signal_connect (model, "toolbar_removed",
-		    G_CALLBACK (toolbar_removed_cb), toolbar);
+		    G_CALLBACK (toolbar_removed_cb), etoolbar);
   g_signal_connect (model, "toolbar_changed",
-		    G_CALLBACK (toolbar_changed_cb), toolbar);
+		    G_CALLBACK (toolbar_changed_cb), etoolbar);
+}
+
+static void
+egg_editable_toolbar_init (EggEditableToolbar *etoolbar)
+{
+  EggEditableToolbarPrivate *priv;
+
+  priv = etoolbar->priv = EGG_EDITABLE_TOOLBAR_GET_PRIVATE (etoolbar);
+
+  priv->save_hidden = TRUE;
+}
+
+static void
+egg_editable_toolbar_dispose (GObject *object)
+{
+  EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR (object);
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+
+  if (priv->fixed_toolbar != NULL)
+    {
+      g_object_unref (priv->fixed_toolbar);
+      priv->fixed_toolbar = NULL;
+    }
+
+  if (priv->manager != NULL)
+    {
+      g_object_unref (priv->manager);
+      priv->manager = NULL;
+    }
+
+  if (priv->model)
+    {
+      egg_editable_toolbar_disconnect_model (etoolbar);
+      g_object_unref (priv->model);
+      priv->model = NULL;
+    }
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 egg_editable_toolbar_set_ui_manager (EggEditableToolbar *etoolbar,
 				     GtkUIManager       *manager)
 {
-  g_return_if_fail (GTK_IS_UI_MANAGER (manager));
-
+  static const GtkActionEntry actions[] = {
+    { "MoveToolItem", NULL, N_("_Move on Toolbar"), NULL,
+      N_("Move the selected item on the toolbar"), G_CALLBACK (move_item_cb) },
+    { "RemoveToolItem", GTK_STOCK_REMOVE, N_("_Remove from Toolbar"), NULL,
+      N_("Remove the selected item from the toolbar"), G_CALLBACK (remove_item_cb) },
+    { "RemoveToolbar", GTK_STOCK_REMOVE, N_("_Remove Toolbar"), NULL,
+      N_("Remove the selected toolbar"), G_CALLBACK (remove_toolbar_cb) },
+  };
+  
   etoolbar->priv->manager = g_object_ref (manager);
+
+  etoolbar->priv->actions = gtk_action_group_new ("ToolbarActions");
+  gtk_action_group_set_translation_domain (etoolbar->priv->actions, GETTEXT_PACKAGE);
+  gtk_action_group_add_actions (etoolbar->priv->actions, actions,
+		 		G_N_ELEMENTS (actions), etoolbar);
+  gtk_ui_manager_insert_action_group (manager, etoolbar->priv->actions, -1);
+  g_object_unref (etoolbar->priv->actions);
+
+  etoolbar->priv->popup_id = gtk_ui_manager_add_ui_from_string (manager, 
+     "<popup name=\"ToolbarPopup\">"
+     "<menuitem action=\"MoveToolItem\"/>"
+     "<menuitem action=\"RemoveToolItem\"/>"							     
+     "<menuitem action=\"RemoveToolbar\"/>"
+     "</popup>", -1, NULL);  
+  
+  toolbar_visibility_refresh (etoolbar);
+}
+
+GtkWidget * egg_editable_toolbar_get_selected (EggEditableToolbar   *etoolbar)
+{
+  return etoolbar->priv->selected;
+}
+
+void
+egg_editable_toolbar_set_selected (EggEditableToolbar *etoolbar,
+				   GtkWidget          *widget)
+{
+  gboolean toolitem, toolbar;
+
+  etoolbar->priv->selected = widget;
+  
+  toolitem = (gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM) != 0);
+  toolbar = (gtk_widget_get_ancestor (widget, GTK_TYPE_TOOLBAR) != 0);
+  
+  gtk_action_set_visible (find_action (etoolbar, "RemoveToolbar"), toolbar && (etoolbar->priv->edit_mode > 0));
+  gtk_action_set_visible (find_action (etoolbar, "RemoveToolItem"), toolitem);
+  gtk_action_set_visible (find_action (etoolbar, "MoveToolItem"), toolitem);
+  
+  toolbar_visibility_refresh (etoolbar);
 }
 
 static void
@@ -1261,6 +1303,12 @@ egg_editable_toolbar_set_property (GObject      *object,
     case PROP_TOOLBARS_MODEL:
       egg_editable_toolbar_set_model (etoolbar, g_value_get_object (value));
       break;
+    case PROP_SELECTED:
+      egg_editable_toolbar_set_selected (etoolbar, g_value_get_object (value));
+      break;
+    case PROP_VISIBILITY_PATH:
+      etoolbar->priv->visibility_path = g_strdup (g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1283,6 +1331,9 @@ egg_editable_toolbar_get_property (GObject    *object,
     case PROP_TOOLBARS_MODEL:
       g_value_set_object (value, etoolbar->priv->model);
       break;
+    case PROP_SELECTED:
+      g_value_set_object (value, etoolbar->priv->selected);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1296,7 +1347,7 @@ egg_editable_toolbar_class_init (EggEditableToolbarClass *klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  object_class->finalize = egg_editable_toolbar_finalize;
+  object_class->dispose = egg_editable_toolbar_dispose;
   object_class->set_property = egg_editable_toolbar_set_property;
   object_class->get_property = egg_editable_toolbar_get_property;
 
@@ -1322,82 +1373,73 @@ egg_editable_toolbar_class_init (EggEditableToolbarClass *klass)
 							"Toolbars Model",
 							EGG_TYPE_TOOLBARS_MODEL,
 							G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+				   PROP_SELECTED,
+				   g_param_spec_object ("selected",
+							"Selected",
+							"Selected toolitem",
+							GTK_TYPE_TOOL_ITEM,
+							G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+				   PROP_VISIBILITY_PATH,
+				   g_param_spec_string ("visibility-path",
+							"visibility-path",
+							"visibility-path",
+							NULL,
+							G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
   g_type_class_add_private (object_class, sizeof (EggEditableToolbarPrivate));
 }
 
-static void
-egg_editable_toolbar_init (EggEditableToolbar *etoolbar)
+GtkWidget *
+egg_editable_toolbar_new (GtkUIManager *manager,
+                          const char *visibility_path)
 {
-  etoolbar->priv = EGG_EDITABLE_TOOLBAR_GET_PRIVATE (etoolbar);
-  etoolbar->priv->save_hidden = TRUE;
-}
-
-static void
-egg_editable_toolbar_finalize (GObject *object)
-{
-  EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR (object);
-
-  if (etoolbar->priv->fixed_toolbar)
-    {
-      g_object_unref (etoolbar->priv->fixed_toolbar);
-    }
-
-  if (etoolbar->priv->manager)
-    {
-      g_object_unref (etoolbar->priv->manager);
-    }
-
-  if (etoolbar->priv->model)
-    {
-      egg_editable_toolbar_disconnect_model (etoolbar);
-      g_object_unref (etoolbar->priv->model);
-    }
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+    return GTK_WIDGET (g_object_new (EGG_TYPE_EDITABLE_TOOLBAR,
+                                     "ui-manager", manager,
+                                     "visibility-path", visibility_path,
+                                     NULL));
 }
 
 GtkWidget *
-egg_editable_toolbar_new (GtkUIManager     *manager)
+egg_editable_toolbar_new_with_model (GtkUIManager *manager,
+ 				     EggToolbarsModel *model,
+                                     const char *visibility_path)
 {
   return GTK_WIDGET (g_object_new (EGG_TYPE_EDITABLE_TOOLBAR,
-				   "ui-manager", manager,
-				   NULL));
-}
-
-GtkWidget *
-egg_editable_toolbar_new_with_model (GtkUIManager     *manager,
-				     EggToolbarsModel *model)
-{
-  return GTK_WIDGET (g_object_new (EGG_TYPE_EDITABLE_TOOLBAR,
-				   "ui-manager", manager,
-				   "model", model,
+                                   "ui-manager", manager,
+                                   "model", model,
+                                   "visibility-path", visibility_path,
 				   NULL));
 }
 
 gboolean
 egg_editable_toolbar_get_edit_mode (EggEditableToolbar *etoolbar)
 {
-	return (etoolbar->priv->edit_mode > 0);
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+
+  return priv->edit_mode > 0;
 }
 
 void
 egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
-				    gboolean            mode)
+				    gboolean mode)
 {
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
   int i, l, n_items;
 
-  i = etoolbar->priv->edit_mode;
+  i = priv->edit_mode;
   if (mode)
     {
-      etoolbar->priv->edit_mode++;
+      priv->edit_mode++;
     }
   else
     {
-      g_return_if_fail (etoolbar->priv->edit_mode > 0);
-      etoolbar->priv->edit_mode--;
+      g_return_if_fail (priv->edit_mode > 0);
+      priv->edit_mode--;
     }
-  i *= etoolbar->priv->edit_mode;
+  i *= priv->edit_mode;
   
   if (i == 0)
     {
@@ -1408,9 +1450,9 @@ egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
           toolbar = get_toolbar_nth (etoolbar, i);
           n_items = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar));
 
-          if (n_items == 0 && etoolbar->priv->edit_mode == 0)
+          if (n_items == 0 && priv->edit_mode == 0)
             {
-              egg_toolbars_model_remove_toolbar (etoolbar->priv->model, i);
+              egg_toolbars_model_remove_toolbar (priv->model, i);
             }
           else
             {          
@@ -1430,12 +1472,11 @@ egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
 
 void
 egg_editable_toolbar_show (EggEditableToolbar *etoolbar,
-			   const char         *name)
+			   const char *name)
 {
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  EggToolbarsModel *model = priv->model;
   int i, n_toolbars;
-  EggToolbarsModel *model = etoolbar->priv->model;
-
-  g_return_if_fail (model != NULL);
 
   n_toolbars = egg_toolbars_model_n_toolbars (model);
   for (i = 0; i < n_toolbars; i++)
@@ -1444,20 +1485,19 @@ egg_editable_toolbar_show (EggEditableToolbar *etoolbar,
 
       toolbar_name = egg_toolbars_model_toolbar_nth (model, i);
       if (strcmp (toolbar_name, name) == 0)
-      {
-        gtk_widget_show (get_dock_nth (etoolbar, i));
-      }
+        {
+          gtk_widget_show (get_dock_nth (etoolbar, i));
+        }
     }
 }
 
 void
 egg_editable_toolbar_hide (EggEditableToolbar *etoolbar,
-			   const char         *name)
+			   const char *name)
 {
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
+  EggToolbarsModel *model = priv->model;
   int i, n_toolbars;
-  EggToolbarsModel *model = etoolbar->priv->model;
-
-  g_return_if_fail (model != NULL);
 
   n_toolbars = egg_toolbars_model_n_toolbars (model);
   for (i = 0; i < n_toolbars; i++)
@@ -1473,28 +1513,29 @@ egg_editable_toolbar_hide (EggEditableToolbar *etoolbar,
 }
 
 void
-egg_editable_toolbar_set_fixed (EggEditableToolbar *toolbar,
-				GtkToolbar         *fixed_toolbar)
+egg_editable_toolbar_set_fixed (EggEditableToolbar *etoolbar,
+				GtkToolbar *toolbar)
 {
-  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (toolbar));
-  g_return_if_fail (!fixed_toolbar || GTK_IS_TOOLBAR (fixed_toolbar));
+  EggEditableToolbarPrivate *priv = etoolbar->priv;
 
-  if (toolbar->priv->fixed_toolbar)
+  g_return_if_fail (!toolbar || GTK_IS_TOOLBAR (toolbar));
+
+  if (priv->fixed_toolbar)
     {
-      unparent_fixed (toolbar);
-      g_object_unref (toolbar->priv->fixed_toolbar);
-      toolbar->priv->fixed_toolbar = NULL;
+      unparent_fixed (etoolbar);
+      g_object_unref (priv->fixed_toolbar);
+      priv->fixed_toolbar = NULL;
     }
 
-  if (fixed_toolbar)
+  if (toolbar)
     {
-      toolbar->priv->fixed_toolbar = GTK_WIDGET (fixed_toolbar);
-      gtk_toolbar_set_show_arrow (fixed_toolbar, FALSE);
-      g_object_ref (fixed_toolbar);
-      gtk_object_sink (GTK_OBJECT (fixed_toolbar));
+      priv->fixed_toolbar = GTK_WIDGET (toolbar);
+      gtk_toolbar_set_show_arrow (toolbar, FALSE);
+      g_object_ref (toolbar);
+      gtk_object_sink (GTK_OBJECT (toolbar));
     }
 
-  update_fixed (toolbar);
+  update_fixed (etoolbar);
 }
 
 #define DEFAULT_ICON_HEIGHT 20
