@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
  *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2004 Christian Persch
+ *  Copyright (C) 2004, 2005, 2006 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.37 2005/12/30 19:28:11 chpe Exp $
+ *  $Id: ephy-file-helpers.c,v 1.45 2006/01/29 14:30:15 chpe Exp $
  */
 
 #include "config.h"
@@ -28,39 +28,45 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/stat.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libxml/xmlreader.h>
 
 /* bug http://bugzilla.gnome.org/show_bug.cgi?id=156687 */
 #undef GNOME_DISABLE_DEPRECATED
 #include <libgnome/gnome-desktop-item.h>
 
-#ifdef HAVE_STARTUP_NOTIFICATION
 #define SN_API_NOT_YET_FROZEN
 #include <libsn/sn.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
-#endif
+
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <sys/stat.h>
 
 #define EPHY_UUID		"0d82d98f-7079-401c-abff-203fcde1ece3"
 #define EPHY_UUID_ENVVAR	"EPHY_UNIQUE"
 #define EPHY_UUID_ENVSTRING	EPHY_UUID_ENVVAR "=" EPHY_UUID
 
+#define DELAY_MAX_TICKS	64
+#define INITIAL_TICKS	2
+
 static GHashTable *files = NULL;
 static GHashTable *mime_table = NULL;
 
+static gboolean have_private_profile = FALSE;
 static char *dot_dir = NULL;
 static char *tmp_dir = NULL;
 static GList *del_on_exit = NULL;
 
+GQuark ephy_file_helpers_error_quark;
+
 const char *
 ephy_file_tmp_dir (void)
 {
@@ -247,19 +253,12 @@ ephy_file (const char *filename)
 const char *
 ephy_dot_dir (void)
 {
-	if (dot_dir == NULL)
-	{
-		dot_dir = g_build_filename (g_get_home_dir (),
-					    GNOME_DOT_GNOME,
-					    "epiphany",
-					    NULL);
-	}
-
 	return dot_dir;
 }
 
-void
-ephy_file_helpers_init (void)
+gboolean
+ephy_file_helpers_init (gboolean private_profile,
+			GError **error)
 {
 	const char *uuid;
 
@@ -274,10 +273,40 @@ ephy_file_helpers_init (void)
 	/* Put marker in env */
 	g_setenv (EPHY_UUID_ENVVAR, EPHY_UUID, TRUE);
 
+	ephy_file_helpers_error_quark = g_quark_from_static_string ("ephy-file-helpers-error");
+
 	files = g_hash_table_new_full (g_str_hash,
 				       g_str_equal,
 				       (GDestroyNotify) g_free,
 				       (GDestroyNotify) g_free);
+
+	have_private_profile = private_profile;
+
+	if (private_profile)
+	{
+		if (ephy_file_tmp_dir () == NULL)
+		{
+			g_set_error (error,
+				     EPHY_FILE_HELPERS_ERROR_QUARK,
+				     0,
+				     _("Could not create a temporary directory in “%s”."),
+				     g_get_tmp_dir ());
+			return FALSE;
+		}
+
+		dot_dir = g_build_filename (ephy_file_tmp_dir (),
+					    "epiphany",
+					    NULL);
+	}
+	else
+	{
+		dot_dir = g_build_filename (g_get_home_dir (),
+					    GNOME_DOT_GNOME,
+					    "epiphany",
+					    NULL);
+	}
+	
+	return ephy_ensure_dir_exists (ephy_dot_dir (), error);
 }
 
 static void
@@ -319,21 +348,29 @@ ephy_file_helpers_shutdown (void)
 }
 
 gboolean
-ephy_ensure_dir_exists (const char *dir)
+ephy_ensure_dir_exists (const char *dir,
+		        GError **error)
 {
-	if (g_file_test (dir, G_FILE_TEST_IS_DIR) == FALSE)
+	if (g_file_test (dir, G_FILE_TEST_EXISTS) &&
+	    !g_file_test (dir, G_FILE_TEST_IS_DIR))
 	{
-		if (g_file_test (dir, G_FILE_TEST_EXISTS) == TRUE)
-		{
-			g_warning (_("\"%s\" exists, please move it out of the way."), dir);
-			return FALSE;
-		}
+		g_set_error (error,
+			     EPHY_FILE_HELPERS_ERROR_QUARK,
+			     0,
+			     _("The file “%s” exists. Please move it out of the way."),
+			     dir);
+		return FALSE;
+	}
 
-		if (mkdir (dir, 488) != 0)
-		{
-			g_warning (_("Failed to create directory \"%s\"."), dir);
-			return FALSE;
-		}
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS) &&
+            mkdir (dir, 488) != 0)
+	{
+		g_set_error (error,
+			     EPHY_FILE_HELPERS_ERROR_QUARK,
+			     0,
+			     _("Failed to create directory “%s”."),
+			     dir);
+		return FALSE;
 	}
 
 	return TRUE;
@@ -560,7 +597,6 @@ my_gdk_spawn_make_environment_for_screen (GdkScreen  *screen,
   return retval;
 }
 
-#ifdef HAVE_STARTUP_NOTIFICATION
 static void
 sn_error_trap_push (SnDisplay *display,
 		    Display   *xdisplay)
@@ -728,8 +764,6 @@ add_startup_timeout (GdkScreen         *screen,
 	}
 }
 
-#endif /* HAVE_STARTUP_NOTIFICATION */
-
 gboolean
 ephy_file_launch_application (GnomeVFSMimeApplication *application,
 			      const char *parameter,
@@ -740,10 +774,8 @@ ephy_file_launch_application (GnomeVFSMimeApplication *application,
 	char            *uri;
 	char           **envp;
 	GnomeVFSResult   result;
-#ifdef HAVE_STARTUP_NOTIFICATION
 	SnLauncherContext *sn_context;
 	SnDisplay *sn_display;
-#endif
 
 	g_return_val_if_fail (application != NULL, FALSE);
 	g_return_val_if_fail (parameter != NULL, FALSE);
@@ -756,7 +788,6 @@ ephy_file_launch_application (GnomeVFSMimeApplication *application,
 	screen = gdk_screen_get_default ();
 	envp = my_gdk_spawn_make_environment_for_screen (screen, NULL);
 	
-#ifdef HAVE_STARTUP_NOTIFICATION
 	sn_display = sn_display_new (gdk_display,
 				     sn_error_trap_push,
 				     sn_error_trap_pop);
@@ -807,11 +838,9 @@ ephy_file_launch_application (GnomeVFSMimeApplication *application,
 	} else {
 		sn_context = NULL;
 	}
-#endif /* HAVE_STARTUP_NOTIFICATION */
 	
 	result = gnome_vfs_mime_application_launch_with_env (application, uris, envp);
 
-#ifdef HAVE_STARTUP_NOTIFICATION
 	if (sn_context != NULL) {
 		if (result != GNOME_VFS_OK) {
 			sn_launcher_context_complete (sn_context); /* end sequence */
@@ -824,7 +853,6 @@ ephy_file_launch_application (GnomeVFSMimeApplication *application,
 	}
 	
 	sn_display_unref (sn_display);
-#endif /* HAVE_STARTUP_NOTIFICATION */
 
 	g_strfreev (envp);
 	g_list_foreach (uris, (GFunc) g_free,NULL);
@@ -964,3 +992,163 @@ ephy_file_launch_handler (const char *mime_type,
 
 	return ret;
 }
+
+struct _EphyFileMonitor
+{
+	GnomeVFSMonitorHandle *handle;
+	EphyFileMonitorFunc callback;
+	EphyFileMonitorDelayFunc delay_func;
+	gpointer user_data;
+	char *uri;
+	guint delay;
+	guint timeout_id;
+	guint ticks;
+	GnomeVFSMonitorEventType type;
+};
+
+static gboolean
+ephy_file_monitor_timeout_cb (EphyFileMonitor *monitor)
+{
+	if (monitor->ticks > 0)
+	{
+		monitor->ticks--;
+
+		/* Run again */
+		return TRUE;
+	}
+
+	if (monitor->delay_func &&
+	    monitor->delay_func (monitor, monitor->user_data))
+	{
+		monitor->ticks = DELAY_MAX_TICKS / 2;
+
+		/* Run again */
+		return TRUE;
+	}
+
+	monitor->timeout_id = 0;
+
+	monitor->callback (monitor, monitor->uri, monitor->type, monitor->user_data);
+
+	/* don't run again */
+	return FALSE;
+}
+
+static void
+ephy_file_monitor_cb (GnomeVFSMonitorHandle *handle,
+		      const char *monitor_uri,
+		      const char *info_uri,
+		      GnomeVFSMonitorEventType event_type,
+		      EphyFileMonitor *monitor)
+{
+	LOG ("File '%s' has changed, scheduling reload", monitor_uri);
+
+	switch (event_type)
+	{
+		case GNOME_VFS_MONITOR_EVENT_CHANGED:
+			monitor->ticks = INITIAL_TICKS;
+			/* fall-through */
+		case GNOME_VFS_MONITOR_EVENT_CREATED:
+			/* We make a lot of assumptions here, but basically we know
+			 * that we just have to reload, by construction.
+			 * Delay the reload a little bit so we don't endlessly
+			 * reload while a file is written.
+			 */
+			monitor->type = event_type;
+
+			if (monitor->ticks == 0)
+			{
+				monitor->ticks = 1;
+			}
+			else
+			{
+				/* Exponential backoff */
+				monitor->ticks = MIN (monitor->ticks * 2,
+						      DELAY_MAX_TICKS);
+			}
+
+			if (monitor->timeout_id == 0)
+			{
+				monitor->timeout_id = 
+					g_timeout_add (monitor->delay,
+						       (GSourceFunc) ephy_file_monitor_timeout_cb,
+						       monitor);
+			}
+
+			break;
+
+		case GNOME_VFS_MONITOR_EVENT_DELETED:
+			if (monitor->timeout_id != 0)
+			{
+				g_source_remove (monitor->timeout_id);
+				monitor->timeout_id = 0;
+			}
+			monitor->ticks = 0;
+
+			monitor->callback (monitor, monitor->uri, event_type, monitor->user_data);
+			break;
+		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
+		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
+		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
+		default:
+			break;
+	}
+}
+
+EphyFileMonitor *
+ephy_file_monitor_add (const char *uri,
+		       GnomeVFSMonitorType monitor_type,
+		       guint delay,
+		       EphyFileMonitorFunc callback,
+		       EphyFileMonitorDelayFunc delay_func,
+		       gpointer user_data)
+{
+	EphyFileMonitor *monitor;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (callback, NULL);
+
+	monitor = g_new (EphyFileMonitor, 1);
+	monitor->callback = callback;
+	monitor->delay_func = delay_func;
+	monitor->user_data = user_data;
+	monitor->uri = g_strdup (uri);
+	monitor->delay = delay;
+	monitor->ticks = 0;
+	monitor->timeout_id = 0;
+
+	if (gnome_vfs_monitor_add (&monitor->handle, uri, monitor_type,
+				   (GnomeVFSMonitorCallback) ephy_file_monitor_cb,
+				   monitor) != GNOME_VFS_OK)
+	{
+		LOG ("Failed to add file monitor for '%s'", uri);
+
+		g_free (monitor->uri);
+		g_free (monitor);
+		return NULL;
+	}
+
+	LOG ("File monitor for '%s' added", uri);
+
+	return monitor;
+}
+
+void
+ephy_file_monitor_cancel (EphyFileMonitor *monitor)
+{
+	g_return_if_fail (monitor != NULL);
+	g_return_if_fail (monitor->handle != NULL);
+	g_return_if_fail (monitor->uri != NULL);
+
+	LOG ("Cancelling file monitor for '%s'", monitor->uri);
+
+	gnome_vfs_monitor_cancel (monitor->handle);
+
+	if (monitor->timeout_id != 0)
+	{
+		g_source_remove (monitor->timeout_id);
+	}
+
+	g_free (monitor->uri);
+	g_free (monitor);
+}
