@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-menu.c,v 1.44 2006/01/16 23:38:26 paharvey Exp $
+ *  $Id: ephy-bookmarks-menu.c,v 1.46 2006/01/17 22:56:59 paharvey Exp $
  */
 
 #include "config.h"
@@ -252,12 +252,17 @@ ephy_bookmarks_menu_build (GString *string, EphyNode *parent)
 	/* Otherwise, build the menu with "Open in tabs". */
 	else
 	{
+		char *name;
+
 		append_menu (string, topics, children, flags);
 		g_ptr_array_free (topics, TRUE);
 	
-		char *name = ephy_open_tabs_action_name (node);
-		g_string_append_printf
-		  (string, "<separator/><menuitem action=\"%s\" name=\"OpenTabs\"/>", name);
-		g_free (name);
+		if (children->len > 1)
+		{
+			name = ephy_open_tabs_action_name (node);
+			g_string_append_printf
+			  (string, "<separator/><menuitem action=\"%s\" name=\"OpenTabs\"/>", name);
+			g_free (name);
+		}
 	}
 }
