@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.h,v 1.37 2006/01/16 23:03:27 paharvey Exp $
+ *  $Id: ephy-bookmarks.h,v 1.40 2006/01/30 23:02:35 paharvey Exp $
  */
 
 #ifndef EPHY_BOOKMARKS_H
@@ -88,6 +88,11 @@ EphyNode	 *ephy_bookmarks_add			(EphyBookmarks *eb,
 EphyNode*	  ephy_bookmarks_find_bookmark		(EphyBookmarks *eb,
 							 const char *url);
 
+gint              ephy_bookmarks_get_similar		(EphyBookmarks *eb,
+							 EphyNode *bookmark,
+							 GPtrArray *identical,
+							 GPtrArray *similar);
+    
 void		  ephy_bookmarks_set_icon		(EphyBookmarks *eb,
 							 const char *url,
 							 const char *icon);
