@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history-window.c,v 1.101 2006/01/16 23:02:50 paharvey Exp $
+ *  $Id: ephy-history-window.c,v 1.102 2006/01/27 22:14:43 paharvey Exp $
  */
 
 #include "config.h"
@@ -489,7 +489,7 @@ cmd_bookmark_link (GtkAction *action,
 		location = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 		title = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_TITLE);
 
-		ephy_bookmarks_ui_add_bookmark (GTK_WIDGET (editor), location, title);
+		ephy_bookmarks_ui_add_bookmark (location, title);
 	}
 
 	g_list_free (selection);
